#define KeyPointCount 20
#define MaxFace 2
#define MaxFace_KeyPointCount (KeyPointCount*MaxFace)

precision highp float;
attribute vec2 aVertexCo;
varying vec2 textureCoordinate;
uniform vec4 u_centers_radius[MaxFace_KeyPointCount];
uniform vec4 u_scales_angles_mins_maxs[MaxFace_KeyPointCount];
uniform vec2 u_types_intensities[MaxFace_KeyPointCount];
uniform float u_counts[MaxFace];
uniform float u_faceRoll[MaxFace];
uniform float u_aspectRatio;

void main() {
    gl_Position = vec4(aVertexCo * 2.0 - 1.0, 0.0, 1.0);
    vec2 uv = aVertexCo;

    for (int face = 0; face < MaxFace; face++) {
        if(u_counts[face] < 0.5){
            continue;
        }
        for (int i = 0; i < ParamSize; i++) {
            int pointIdx = i+face*KeyPointCount;
            if(u_scales_angles_mins_maxs[pointIdx].x == 0.0 || u_types_intensities[pointIdx].x == 0.0) {
                continue;
            }
            vec2 textureCoordinateToUse = uv;
            
            vec2 coordinateFromCenter = textureCoordinateToUse - u_centers_radius[pointIdx].xy;
            float coordinateWithRotateX = coordinateFromCenter.x*cos(u_faceRoll[face]) - coordinateFromCenter.y*sin(u_faceRoll[face])*u_aspectRatio;
            float coordinateWithRotateY = coordinateFromCenter.x*sin(u_faceRoll[face])/u_aspectRatio + coordinateFromCenter.y*cos(u_faceRoll[face]);
            float e1 = ( coordinateWithRotateX ) / ( u_centers_radius[pointIdx].z );
            float e2 = ( coordinateWithRotateY ) / ( u_centers_radius[pointIdx].w  / u_aspectRatio );
            float d = (e1 * e1) + (e2 * e2);
            
            if (d < 1.0) {
                if(u_types_intensities[pointIdx].x == 1.0) {
                    vec2 dist = vec2(d * u_centers_radius[pointIdx].z, d * u_centers_radius[pointIdx].w);
                    textureCoordinateToUse -= u_centers_radius[pointIdx].xy;
                    vec2 delta = ((u_centers_radius[pointIdx].zw - dist) / u_centers_radius[pointIdx].zw);
                    float deltaScale = u_scales_angles_mins_maxs[pointIdx].x;
                    if(deltaScale > 0.0) {
                        deltaScale = smoothstep(u_scales_angles_mins_maxs[pointIdx].z, u_scales_angles_mins_maxs[pointIdx].w, deltaScale);
                    }
                    vec2 percent = 1.0 - ((delta * deltaScale) * u_types_intensities[pointIdx].y);
                    textureCoordinateToUse = textureCoordinateToUse * percent;
                    uv = textureCoordinateToUse + u_centers_radius[pointIdx].xy;
                }
                if(u_types_intensities[pointIdx].x == 2.0) {
                    float dist = 1.0 - d;
                    float delta = u_scales_angles_mins_maxs[pointIdx].x * dist * u_types_intensities[pointIdx].y;
                    float deltaScale = smoothstep(u_scales_angles_mins_maxs[pointIdx].z,
                        u_scales_angles_mins_maxs[pointIdx].w, dist);
                    float directionX = cos(u_scales_angles_mins_maxs[pointIdx].y) * deltaScale;
                    float directionY = sin(u_scales_angles_mins_maxs[pointIdx].y) * deltaScale / (3.0/4.0 * u_aspectRatio);
                    uv = vec2(textureCoordinateToUse.x - (delta * directionX), textureCoordinateToUse.y - (delta * directionY));
                }
            }
        }
    }
    textureCoordinate = uv;
}