precision highp float;
varying vec2 vTextureCoord;
uniform sampler2D sTexture;
uniform sampler2D sTexture1;
uniform sampler2D sTexture2;
uniform float brightness;
uniform float vivid;
float sharpIntensity = 1.365;
float lookupIntensity = 0.6;
vec3 skinColor = vec3(0.882353,0.686275,0.439216);
const float stepDis = 1.0 / 4.0;
const float perPixel = 1.0 / 64.0;
const float halfPixel = 0.5 / 64.0;



const vec3 LumCoeff = vec3(0.2125, 0.7154, 0.0721);
const vec3 AvgLumin = vec3(0.5, 0.5, 0.5);
vec3 ContrastSaturationBrightness(vec3 color, float brt, float sat, float con) {
    vec3 brtColor = color * brt;
    vec3 intensity = vec3(dot(brtColor, LumCoeff));
    vec3 satColor = mix(intensity, brtColor, sat);
    vec3 conColor = mix(AvgLumin, satColor, con);
    return conColor;
}

vec3 lookup2(vec3 color) {
    float blue = color.b * 15.0;
    vec2 coord1;
    coord1.y = floor(blue / 4.0);
    coord1.x = floor(blue) - (coord1.y * 4.0);
    coord1 = coord1 * stepDis + halfPixel + (stepDis - perPixel) * color.xy;
    vec3 newColor = texture2D(sTexture2, coord1).rgb;
    return mix(color, newColor, lookupIntensity);
}

vec3 lookup(vec3 textureColor)
{
     float blueColor = textureColor.b * 15.0;

     vec2 quad1;
     quad1.y = floor(floor(blueColor) / 4.0);
     quad1.x = floor(blueColor) - (quad1.y * 4.0);

     vec2 quad2;
     quad2.y = floor(ceil(blueColor) / 4.0);
     quad2.x = ceil(blueColor) - (quad2.y * 4.0);

     highp vec2 texPos1;
     texPos1.x = (quad1.x * 0.25) + 0.5/64.0 + ((0.25 - 1.0/64.0) * textureColor.r);
     texPos1.y = (quad1.y * 0.25) + 0.5/64.0 + ((0.25 - 1.0/64.0) * textureColor.g);

     highp vec2 texPos2;
     texPos2.x = (quad2.x * 0.25) + 0.5/64.0 + ((0.25 - 1.0/64.0) * textureColor.r);
     texPos2.y = (quad2.y * 0.25) + 0.5/64.0 + ((0.25 - 1.0/64.0) * textureColor.g);

     lowp vec4 newColor1 = texture2D(sTexture2, texPos1);
     lowp vec4 newColor2 = texture2D(sTexture2, texPos2);

     lowp vec4 newColor = mix(newColor1, newColor2, fract(blueColor));
     lowp vec3 result = mix(textureColor, newColor.rgb, lookupIntensity);
     return result;
}


void main() {
    vec4 sharpImageColor = texture2D(sTexture, vTextureCoord);
    vec3 blurredImageColor = texture2D(sTexture1, vTextureCoord).rgb;
    gl_FragColor = vec4(clamp(mix(blurredImageColor, sharpImageColor.rgb, sharpIntensity), 0.0, 1.0), sharpImageColor.a);

    float weight = 0.6;

    gl_FragColor.rgb = lookup(gl_FragColor.rgb);

    gl_FragColor.rgb = mix(sharpImageColor.rgb, gl_FragColor.rgb, gl_FragColor.a * weight);

    gl_FragColor.rgb = ContrastSaturationBrightness(gl_FragColor.rgb, 1.0, vivid, 1.0);

    gl_FragColor.rgb = (1.0 - brightness) *  gl_FragColor.rgb + brightness * sin(1.57 *  gl_FragColor.rgb);
}