precision highp float;
varying vec2 vTextureCoord;
uniform sampler2D sTexture;
uniform sampler2D sTexture1;

uniform float filterOpacity;
const float factor1 = 2.782;
const float factor2 = 1.131;
const float factor3 = 1.158;
const float factor4 = 2.901;
const float factor5 = 0.979;
const float factor6 = 0.639;
const float factor7 = 0.963;
const float blurOpacity = 0.46;

const vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
const float e = 1.0e-10;

vec3 rgb2h_v(vec3 c) {
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    float d = q.x - min(q.w, q.y);
    vec3 h_v = vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), 0.0, q.x);
    return h_v;
}

const vec3 LumCoeff = vec3(0.2125, 0.7154, 0.0721);
const vec3 AvgLumin = vec3(0.5, 0.5, 0.5);

vec3 ContrastSaturationBrightness(vec3 color, float brt, float sat, float con) {
    vec3 brtColor = color * brt;
    vec3 intensity = vec3(dot(brtColor, LumCoeff));
    vec3 satColor = mix(intensity, brtColor, sat);
    vec3 conColor = mix(AvgLumin, satColor, con);
    return conColor;
}

const vec3 W = vec3(0.0);

void main() {
    vec4 inputColor = texture2D(sTexture, vTextureCoord);
    vec3 hsv = rgb2h_v(inputColor.rgb);
    float opacityLimit = 1.0;
    if ((0.18 <= hsv.x && hsv.x <= 0.89) || hsv.z <= 0.2) {
        gl_FragColor = inputColor; return;
    }
    if (0.16 < hsv.x && hsv.x < 0.18) {
        opacityLimit = min(opacityLimit, (0.18 - hsv.x) / 0.02);
    }
    if (0.89 < hsv.x && hsv.x < 0.91) {
        opacityLimit = min(opacityLimit, 1.0 - (0.91 - hsv.x) / 0.02);
    }
    if (0.2 < hsv.z && hsv.z < 0.3) {
        opacityLimit = min(opacityLimit, 1.0 - (0.3 - hsv.z) / 0.1);
    }
    vec4 blurColor = texture2D(sTexture1, vTextureCoord);
    opacityLimit = blurOpacity * opacityLimit;
    float cDistance = distance(W, max(blurColor.rgb - inputColor.rgb, 0.0)) * factor1;
    vec3 brightColor = ContrastSaturationBrightness(inputColor.rgb, factor2, 1.0, factor3);
    vec3 mix11Color = mix(inputColor.rgb, brightColor.rgb, cDistance);
    float dDistance = distance(W, max(inputColor.rgb-blurColor.rgb, 0.0)) * factor4;
    vec3 darkColor = ContrastSaturationBrightness(inputColor.rgb, factor5, 1.0, factor6);
    vec3 mix115Color = mix(mix11Color.rgb, darkColor.rgb, dDistance);
    vec3 mix12Color;
    vec3 mix116Color = mix(inputColor.rgb, mix115Color.rgb, factor7);
    mix12Color = mix(mix116Color.rgb, blurColor.rgb, opacityLimit);
    gl_FragColor = vec4(mix(inputColor.rgb, mix12Color.rgb, filterOpacity), 1.0);
}