// define 以下值：
// CONTROL_POINT_SIZE
attribute vec4 aVertexCo;
uniform mat4 World;

uniform float aspectRatio;
uniform vec4 info0[CONTROL_POINT_SIZE];
uniform vec4 info1[CONTROL_POINT_SIZE];
uniform vec4 info2[CONTROL_POINT_SIZE];

varying vec2 textureCoordinate;

vec2 cal_coord(){
    vec2 textureCoordinateToUse = vec2(aVertexCo.x, (aVertexCo.y * aspectRatio + 0.5 - 0.5 * aspectRatio));
    for(int i = 0; i < CONTROL_POINT_SIZE; i++){
        if(info2[i].w != 0.0){
            vec2 center = info0[i].xy;
            // radius 信息为椭圆的长短轴，x轴为rx_idx1和rx_idx2距离乘以rx_scale。y轴为ry_idx1和ry_idx2距离乘以ry_scale
            vec2 radius = info0[i].zw;
            // Scale 信息直接使用输入中的scale_x和scale_y
            vec2 scale = info1[i].xy;
            // MinMax 信息直接使用输入中的range_min和range_max
            float uMin = info1[i].z;
            float uMax = info1[i].w;
            // Sign 信息直接使用输入中的sign_x和sign_y
            vec2 signFlag = info2[i].xy;
            // angle为人脸roll
            float angle = info2[i].z;
            //计算变形的纹理参考中心
            vec2 centerToUse = vec2(center.x, (center.y * aspectRatio + 0.5 - 0.5 * aspectRatio));
            vec2 distv = textureCoordinateToUse - centerToUse;
            mat2 rotMp = mat2(cos(angle),sin(angle),-sin(angle),cos(angle));
            distv = distv*rotMp;
            float dist = distv.x*distv.x/(radius.x*radius.x) + distv.y*distv.y/(radius.y*radius.y);
            if (dist < 1.0)
            {
                float center_range = (uMin + uMax)/2.0;
                vec2 p1 = scale*smoothstep(uMin, center_range, dist);
                vec2 p2 = scale*smoothstep(center_range, uMax, dist);
                vec2 percent = 1.0-p1+p2;
                vec2 flag = sign(signFlag*distv);
                flag = 1.0-length(sign(signFlag)) + flag*(flag+1.0)*0.5;
                percent = (percent-1.0) * flag + 1.0;
                percent = percent * percent;
                distv = distv*percent;
                distv = vec2(distv.x*cos(angle)-distv.y*sin(angle), distv.x*sin(angle)+distv.y*cos(angle));
                textureCoordinateToUse = distv + centerToUse;
            }
        }
    }
    return vec2(textureCoordinateToUse.x, (0.5+(textureCoordinateToUse.y-0.5)/aspectRatio));
}

void main()
{
    gl_Position = World * vec4(aVertexCo.xy * 2.0 - 1.0, 0.0, 1.);
    textureCoordinate = cal_coord();

}