// define 以下值：
// CONTROL_POINT_SIZE
attribute vec4 aVertexCo;
uniform mat4 World;

uniform float aspectRatio;
uniform vec4 info0[CONTROL_POINT_SIZE];
uniform vec4 info1[CONTROL_POINT_SIZE];
uniform vec4 info2[CONTROL_POINT_SIZE];
uniform vec4 info3[CONTROL_POINT_SIZE];
uniform vec4 info4[CONTROL_POINT_SIZE];

varying vec2 textureCoordinate;


vec2 cal_coord(){
    vec2 textureCoordinateToUse = vec2(aVertexCo.x, (aVertexCo.y * aspectRatio + 0.5 - 0.5 * aspectRatio));
    for(int i = 0; i < CONTROL_POINT_SIZE; i++){
        if(info2[i].w != 0.0){
            highp vec2 center = info0[i].xy;
            highp vec2 radius = info0[i].zw;
            highp vec2 scale = info1[i].xy;
            highp float uMin = info1[i].z;
            highp float uMax = info1[i].w;
            highp vec2 signFlag = info2[i].xy;
            highp float angle = info2[i].z;
            highp vec2 centerToUse = vec2(center.x, (center.y * aspectRatio + 0.5 - 0.5 * aspectRatio));
            highp vec2 distv = textureCoordinateToUse - centerToUse;
            highp mat2 rotMp = mat2(cos(angle),sin(angle),-sin(angle),cos(angle));
            distv = distv*rotMp;
            highp float dist = distv.x*distv.x/(radius.x*radius.x) + distv.y*distv.y/(radius.y*radius.y);
            if (dist < 1.0)
            {
                highp vec2 flag = sign(signFlag*distv);
                flag = 1.0-length(sign(signFlag)) + flag*(flag+1.0)*0.5;
                highp vec2 newCenter = vec2(0.0, 0.0);
                if(uMin > 0.0) {
                    newCenter = distv*uMin*inversesqrt(dist);
                    distv = distv - newCenter;
                }
                dist = smoothstep(uMin, uMax, sqrt(dist));
                dist = dist*dist;

                if(dist > 0.0) {
                    highp vec2 percent;
                    highp vec4 inverse_curve = 1.0/info4[i];
                    highp vec4 sp = 1.0 - pow(1.0-vec4(scale,scale), inverse_curve);
                    highp vec2 s0 = 1.0 - pow(1.0-info3[i].xy, inverse_curve.xy);
                    if(dist >= info3[i].z) {
                        percent.x = 1.0-sp.z+sp.z*(dist-info3[i].z)/(1.0-info3[i].z);
                        percent.x = pow(percent.x, info4[i].z);
                    } else {
                        percent.x = 1.0-s0.x+dist*(s0.x-sp.x)/info3[i].z;
                        percent.x = pow(percent.x, info4[i].x);
                    }
                    if(dist >= info3[i].w) {
                        percent.y = 1.0-sp.w+sp.w*(dist-info3[i].w)/(1.0-info3[i].w);
                        percent.y = pow(percent.y, info4[i].w);
                    } else {
                        percent.y = 1.0-s0.y+dist*(s0.y-sp.y)/info3[i].w;
                        percent.y = pow(percent.y, info4[i].y);
                    }
                    percent = (percent-1.0) * flag + 1.0;
                    distv = distv*percent;
                    distv = distv + newCenter;
                    distv = vec2(distv.x*cos(angle)-distv.y*sin(angle), distv.x*sin(angle)+distv.y*cos(angle));
                    textureCoordinateToUse = distv + centerToUse;
                }
            }
        }
    }
    return vec2(textureCoordinateToUse.x, (0.5+(textureCoordinateToUse.y-0.5)/aspectRatio));
}

void main()
{
    gl_Position = World * vec4(aVertexCo.xy * 2.0 - 1.0, 0.0, 1.);
    textureCoordinate = cal_coord();

}