precision highp float;
precision highp int;
varying vec2 textureCoordinate;
uniform sampler2D uTexture;
uniform float uWidth;
uniform float uHeight;

//根据公式计算Y分量
float computeYByFormula(float x,float y){
    //获取指定点的纹理像素
    vec4 pixel = texture2D(uTexture, vec2(x,y));
    return pixel.r * 0.2126 + pixel.g * 0.7152 + pixel.b * 0.0722;
}

vec4 averageFourPixel(float x, float y, float dx, float dy){
    //左上角像素坐标
    vec4 pixel0 = texture2D(uTexture, vec2(x, y));

    //右上角像素坐标
    vec4 pixel1 = texture2D(uTexture, vec2(x + dx, y));

    //左下角像素坐标
    vec4 pixel2 = texture2D(uTexture, vec2(x, y + dy));

    //右下角像素坐标
    vec4 pixel3 = texture2D(uTexture, vec2(x + dx, y + dy));
    return (pixel0 + pixel1 + pixel2 + pixel3) / 4.0;
}

float computeUByFormula(float x, float y, float dx, float dy){
    vec4 pixel = averageFourPixel(x, y, dx, dy);
    return -0.09991 * pixel.r - 0.33609 * pixel.g + 0.43600 * pixel.b + 0.5000;
}

float computeVByFormula(float x, float y, float dx, float dy){
    vec4 pixel = averageFourPixel(x, y, dx, dy);
    return 0.61500 * pixel.r - 0.55861 * pixel.g - 0.05639 * pixel.b + 0.5000;
}

vec2 computePos(float t, float bytesPerPixel, float currentYCoord){
    //floor是向下取整
    //计算得出实际的坐标，范围是实际的整数宽高
    vec2 realPos = vec2(floor(uWidth * textureCoordinate.x), floor(uHeight * currentYCoord));

    //mod是求余函数
    //当前的x坐标放大4倍，即前面每个像素都填充了4个Y分量，每个Y分量都占用一个像素，再对宽度求余，得出计算下一个Y分量的像素x坐标
    //同理，y坐标也放大4倍，同时还要加上x坐标的求整，最后乘以t是计算是否隔行读取
    return vec2(mod(realPos.x * bytesPerPixel, uWidth), (realPos.y * bytesPerPixel + floor(realPos.x * bytesPerPixel / uWidth)) * t);
}

vec4 calculateY(){
    //根据当前的y坐标计算应该获取纹理位置
    vec2 pos = computePos(1.0, 4.0, textureCoordinate.y);

    //四维向量全填充Y分量
    vec4 fullYColor = vec4(0);

    //纹理的Y坐标
    float textureYPos = pos.y / uHeight;

    //计算填充四个字节的Y分量
    fullYColor[0] = computeYByFormula(pos.x / uWidth, textureYPos);
    fullYColor[1] = computeYByFormula((pos.x + 1.0) / uWidth, textureYPos);
    fullYColor[2] = computeYByFormula((pos.x + 2.0) / uWidth, textureYPos);
    fullYColor[3] = computeYByFormula((pos.x + 3.0) / uWidth, textureYPos);
    return fullYColor;
}
vec4 calculateUV(float dx,float dy){
    //当前y坐标减去四分之一，即计算Y分量完毕回到0开始计算UV
    //传2进去是因为隔行计算
    vec2 pos = computePos(2.0, 4.0, textureCoordinate.y - 0.2500);

    //四维向量填充两个UV分量
    vec4 doubleUVColor = vec4(0);

    //纹理的Y坐标
    float textureYPos = pos.y / uHeight;

    //计算填充四个字节，两对UV分量
    doubleUVColor[0] = computeUByFormula(pos.x / uWidth, textureYPos, dx, dy);
    doubleUVColor[1] = computeVByFormula(pos.x / uWidth, textureYPos, dx, dy);
    doubleUVColor[2] = computeUByFormula((pos.x + 2.0) / uWidth, textureYPos, dx, dy);
    doubleUVColor[3] = computeVByFormula((pos.x + 2.0) / uWidth, textureYPos, dx, dy);
    return doubleUVColor;
}